@@ -20,6 +20,9 @@
     .Call(C_readDirectory, path, flipY, crop, forceStack, verbosity, labelFormat, singleFile, scanOnly)
 }
 
+# Wrapper function to allow mocking in tests
+.readline <- function (...) base::readline(...)
+
 #' Read one or more DICOM directories
 #' 
 #' These functions are R wrappers around the DICOM-to-NIfTI conversion routines
@@ -52,7 +55,11 @@
 #'   from which file paths will be read.
 #' @param subset If \code{path} is a data frame, an expression which will be
 #'   evaluated in the context of the data frame to determine which series to
-#'   convert. Should evaluate to a logical vector.
+#'   convert. Should evaluate to a logical vector. If \code{path} is a
+#'   character vector, \code{scanDicom} is called on the path(s) first to
+#'   produce the data frame. If this is specified, and does not evaluate to
+#'   \code{NULL}, the read will be noninteractive, irrespective of the value of
+#'   the \code{interactive} argument.
 #' @param flipY If \code{TRUE}, the default, then images will be flipped in the
 #'   Y-axis. This is usually desirable, given the difference between
 #'   orientation conventions in the DICOM and NIfTI-1 formats.
@@ -122,13 +129,23 @@ readDicom <- function (path = ".", subset = NULL, flipY = TRUE, crop = FALSE, fo
     usingTempDirectory <- FALSE
     if (is.data.frame(path))
     {
-        subset <- eval(substitute(subset), path)
+        subset <- as.logical(eval(substitute(subset), path))
         if (!is.null(subset))
             path <- attr(path,"paths")[subset]
         else
             path <- attr(path,"paths")
         usingTempDirectory <- TRUE
     }
+    else if (!missing(subset))
+    {
+        info <- scanDicom(path, forceStack, verbosity, labelFormat)
+        subset <- as.logical(eval(substitute(subset), info))
+        if (!is.null(subset))
+        {
+            path <- attr(info,"paths")[subset]
+            usingTempDirectory <- TRUE
+        }
+    }
     
     results <- lapply(path, function(p) {
         if (usingTempDirectory)
@@ -157,7 +174,7 @@ readDicom <- function (path = ".", subset = NULL, flipY = TRUE, crop = FALSE, fo
             seriesNumbers <- sprintf(paste0("%",digits,"d: "), seq_len(nSeries))
             cat(paste0("\n", seriesNumbers, attr(info,"descriptions")))
             cat("\n\nType <Enter> for all series, 0 for none, or indices separated by spaces or commas")
-            selection <- readline("\nSelected series: ")
+            selection <- .readline("\nSelected series: ")
             if (selection == "")
             {
                 allResults <- .readPath(p, flipY, crop, forceStack, verbosity, labelFormat, FALSE, FALSE)
